/*=============================================================================
 *  AUPresetManager.h
 *  PlayPen
 *-----------------------------------------------------------------------------
 *
 * $Log: AUPresetManager.h,v $
 * Revision 1.1  2004/07/29 00:21:15  luke
 * factored out from PlayPen
 *
 * Revision 1.5  2004/07/28 23:57:20  luke
 * pass 3: final
 *
 * Revision 1.4  2004/07/28 21:43:26  luke
 * 2nd pass: CAFileBrowser API breakup
 *
 * Revision 1.3  2004/05/26 19:42:47  luke
 * don't show factory preset option if unit doesn't have any factory presets
 *
 * Revision 1.2  2004/04/29 23:08:39  luke
 * fix sync w/ interface changes to AUInspector
 *
 * Revision 1.1  2004/04/17 20:05:20  luke
 * source reorg.
 *
 * Revision 1.11  2003/12/02 01:17:19  luke
 * CAFileHandling factored
 *
 * Revision 1.10  2003/10/21 22:03:57  luke
 * change some behavior per discussions with Bill
 *
 * Revision 1.9  2003/10/17 22:39:14  luke
 * implement preset saving
 *
 * Revision 1.8  2003/10/15 16:56:43  luke
 * tighten up code.
 *
 * Revision 1.7  2003/10/15 00:46:04  luke
 * tighten up code
 *
 * Revision 1.6  2003/10/15 00:12:50  luke
 * factor general support into CAFileBrowser class
 *
 * Revision 1.5  2003/10/14 17:21:59  luke
 * now using CAFileHandling class
 *
 * Revision 1.4  2003/09/24 21:55:28  luke
 * fix warnings
 *
 * Revision 1.3  2003/09/24 06:39:37  luke
 * finish implementation of preset browser
 *
 * Revision 1.2  2003/09/23 20:46:44  luke
 * updates
 *
 * Revision 1.1  2003/09/23 00:58:22  luke
 * new file
 *
 *
 *-----------------------------------------------------------------------------
 *  Created by Luke Bellandi on Mon Sep 22 2003.
 *  Copyright (c) 2003 Apple Computer Inc. All rights reserved.
 *=============================================================================*/

#import <Cocoa/Cocoa.h>
#import <AudioUnit/AudioUnit.h>

#import "CAFileBrowser.h"

class CAAUPresetFile;

@interface AUPresetManager : CAFileBrowser {
@private
    AudioUnit				mAudioUnit;
    CAAUPresetFile *		mPresetFile;
    
    NSMutableDictionary *	mFactoryPresets;
}

#pragma mark ____ PUBLIC FUNCTIONS ____
- (void)setAU:(AudioUnit)inAU;

@end
