/*=============================================================================
	CAPrefix.h

	$Log: CAPrefix.h,v $
	Revision 1.1  2003/05/04 00:00:16  jcm10
	first checked in
	
	Revision 0.0  Thu Jul 25 2002 18:33:04 US/Pacific  moorf
	Created
		
	$NoKeywords: $
=============================================================================*/
#if !defined(__CAPrefix_h__)
#define __CAPrefix_h__

//=============================================================================
//	Includes
//=============================================================================

#include <TargetConditionals.h>

//	MacTypes.h has the definitions of basic types like UInt32 and Float64
#if	TARGET_OS_MAC && TARGET_RT_MAC_MACHO
	#include <CoreServices/../Frameworks/CarbonCore.framework/Headers/MacTypes.h>
#else
	#include <MacTypes.h>
#endif

#include <CoreFoundation/CoreFoundation.h>
#include <IOKit/IOKitLib.h>
#include <IOKit/IOMessage.h>
#include <IOKit/audio/IOAudioDefines.h>
#include <IOKit/audio/IOAudioTypes.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#if defined(__cplusplus)
	#include <algorithm>
	#include <iterator>
	#include <list>
	#include <map>
	#include <memory>
	#include <new>
	#include <numeric>
	#include <queue>
	#include <set>
	#include <stack>
	#include <utility>
	#include <vector>
#endif

#endif
